package net.rzdata.demo.dict;

import net.rzdata.demo.dict.domain.Dict;
import net.rzdata.demo.dict.mapper.DictRepository;
import net.rzdata.exception.NotExistDataException;
import net.rzdata.exception.RepeatDataException;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

@Service
public class DictService {

    private final DictRepository dictRepository;

    public DictService(DictRepository dictRepository) {
        this.dictRepository = dictRepository;
    }

    protected void addDict(Dict dict) {
        check(dict);
        dictRepository.get().insert(dict);
    }

    protected List<Dict> getDictByType(String type) {
        return dictRepository.findByTypeOrderBySortAsc(type);
    }

    protected void updateDict(Dict dict) {
        Dict exist = dictRepository.get().selectById(dict.getId());
        if (exist == null) {
            throw new NotExistDataException("字典已被删除，无法更新");
        }
        if (!Objects.equals(dict.getValue(), exist.getValue())) {
            check(dict);
        }
        dictRepository.get().updateById(dict);
        // 如果是字典类 则需要更新该类下所有值的type字段
        if (Objects.equals(exist.getParentId(), Dict.ROOT.getId())) {
            dictRepository.updateType(exist.getType(), dict.getType());
        }
    }

    private Optional<Dict> getDictByTypeAndValue(String type, String value) {
        List<Dict> dictList = this.getDictByType(type);
        return dictList.parallelStream()
            .filter(d -> Objects.equals(d.getValue(), value))
            .findAny();
    }

    private void check(Dict dict) {
        Optional<Dict> exist = this.getDictByTypeAndValue(dict.getType(), dict.getValue());
        if (exist.isPresent()) {
            throw new RepeatDataException(String.format("字典类型:%s, 值:%s 已存在", dict.getType(), dict.getValue()));
        }
    }
}
